$title	climate2.gms	Computation of Climate Response

*	Include the preceding file:

$include climate1

parameter
        m(t)            CO2-equiv concentration billion t
        forc(t)         Radiative forcing - W per m2
        forcoth(t)      Exogenous forcings from other greenhouse gases,
        te(t)           Temperature - atmosphere C
	teref(t)	Reference temperature path
        tl(t)           Temperature - lower ocean C
        termv           Terminal value of atmophere
        deltaE          Difference iterval /0.001/;

set	tfirst(t)	The first time period;

tfirst(t) = yes$(ord(t)=1);

*	Initial conditions for climate model:
m(tfirst) = m0;  te(tfirst) = t0;  tl(tfirst) = tl0;  forcoth(t) = 1.42;

parameter	climate		Climate evolution;
parameter	eref(t)		Reference emissions
		E(t)		Currently estimated emissions path,
		grad		Temperature gradient,
		teinit		Initial temperature path;

*	Save the baseline emissions path:

E(t) = ebau(t);

*	Write out a "subroutine file" for computing the climate model:

$onecho >climatemodel.gms
loop(t,
	m(t)		= 590 + atret*eref(t) + (1-deltam)*(m(t-1)-590) + m0$tfirst(t);
	forc(t)		= 4.1*(LOG(m(t)/590)/LOG(2)) + forcoth(t);
	te(t)		= te(t-1)+c1*(forc(t-1)-lam*te(t-1)-c3*(te(t-1)-tl(t-1))) + t0$tfirst(t);
	tl(t)		= tl(t-1)+c4*(te(t-1)-tl(t-1)) + tl0$tfirst(t);
	teref(t)	= te(t);
);
$offecho

*	Write a second subroutine to compute the gradients:

alias (t,tp);

$onecho >jacobian.gms
	eref(t) = E(t);
$include climatemodel
	teinit(t) = teref(t); 
	grad(t,tp) = 0;	
	loop(tp,eref(tp) = eref(tp) + deltaE;
$include climatemodel
	  grad(t,tp)= (teref(t)-teinit(t)) / deltaE;
	  eref(tp)  = eref(tp) - deltaE;);
	  teref(t)  = teinit(t);

$offecho


parameter	climate		Temperature and abatement policy paths;

*	Compute the climate path and its Jacobian:

$include jacobian

climate(t,"te0") = te(t);
